"""
Set of commands that helps with integration of Alpa
repository.
"""

from pathlib import Path
import re
from typing import Optional, Type

from click import ClickException
import click

from alpa.packit import Packit
from alpa.constants import (
    ALPA_FEAT_BRANCH_PREFIX,
    MAIN_BRANCH,
    PackageRequest,
    DeleteRequest,
    REQUEST_LABEL,
)
from alpa.gh import GithubAPI
from alpa.messages import NO_WRITE_ACCESS_ERR
from alpa.repository.base import LocalRepo, AlpaRepo


class LocalRepoBranch(LocalRepo):
    def __init__(self, repo_path: Path) -> None:
        super().__init__(repo_path)

    @property
    def package(self) -> str:
        return self.branch.removeprefix(ALPA_FEAT_BRANCH_PREFIX)

    def get_history_of_package(self, package: str) -> str:
        raise NotImplementedError("Please implement me!")

    def get_packages(self, regex: str = "") -> list[str]:
        refs_without_main = filter(
            lambda ref: ref != "main", self.get_remote_branches(self.remote_name)
        )
        relevant_refs = self._get_relevant_remote_refs(refs_without_main)

        if regex == "":
            return list(relevant_refs)

        pattern = re.compile(regex)
        return [ref for ref in relevant_refs if pattern.match(ref)]

    def switch_to_package(self, package: str) -> None:
        if self.is_dirty():
            click.secho(
                "Repo is dirty, please commit your changes before switching to"
                f" another package.\n {self.get_status_output()}",
                fg="red",
                err=True,
            )
            return None

        feat_branch = self.get_feat_branch_of_package(package)
        if self.branch_exists(feat_branch) and not self.is_branch_merged(feat_branch):
            branch_to_switch = feat_branch
        else:
            branch_to_switch = package
            if self.branch_exists(feat_branch):
                self.git_cmd(["branch", "-D", feat_branch])

        result = self.git_cmd(["switch", branch_to_switch])
        if result.retval == 0:
            click.echo(result.stderr_and_stdout.replace("branch", "package"))
            return

        process = self.git_cmd(["fetch", self.remote_name, branch_to_switch])
        if process.retval != 0:
            click.secho(f"Package {package} doesn't exist!", fg="red", err=True)
            return

        click.echo(f"Switching to the package {package} for the first time")
        click.echo(
            self.git_cmd(["switch", branch_to_switch]).stderr_and_stdout.replace(
                "branch", "package"
            )
        )

    def _ensure_feature_branch(self) -> None:
        if self.branch != self.package:
            return None

        click.echo("Switching to feature branch")
        self.git_cmd(["switch", "-c", self.feat_branch])

    def create_packit_config(self, override: bool) -> bool:
        packit_conf = Packit(self.package)
        if packit_conf.packit_config_file_exists() and not override:
            return False

        packit_conf.create_packit_config(override)
        return True

    def _rebase_needed(self) -> bool:
        self.git_cmd(["fetch", self.remote_name])
        last_package_commit = self.git_cmd(["rev-parse", self.branch]).stdout
        last_remote_main_commit = self.git_cmd(
            ["rev-parse", f"{self.remote_name}/{MAIN_BRANCH}"]
        ).stdout
        return (
            last_package_commit != last_remote_main_commit
            and self.git_cmd(
                [
                    "merge-base",
                    "--is-ancestor",
                    last_remote_main_commit,
                    last_package_commit,
                ]
            ).retval
            != 0
        )

    def push(self, branch: str, force: bool = False) -> None:
        if self._rebase_needed():
            rebase_from = f"{self.remote_name}/{MAIN_BRANCH}"
            click.secho(
                "Warning! Main branch has new updates! Rebasing your package "
                "with main branch...\n"
                f"git rebase {rebase_from}",
                fg="yellow",
            )
            self.git_cmd(["rebase", rebase_from])

        super().push(branch, force=True)


class AlpaRepoBranch(AlpaRepo, LocalRepoBranch):
    def __init__(self, repo_path: Path, gh_api: Optional[GithubAPI] = None) -> None:
        super().__init__(repo_path, gh_api)

        self.gh_api = gh_api or GithubAPI(self.repo_name)
        self.gh_repo = self.gh_api.get_repo(self.namespace, self.repo_name)

    def create_package(self, package: str) -> None:
        upstream = self.gh_repo.get_upstream()
        if upstream and not upstream.has_write_access(self.gh_api.gh_user):
            raise ClickException(NO_WRITE_ACCESS_ERR)

        self.git_cmd(["switch", MAIN_BRANCH])
        self.git_cmd(["switch", "-c", package])
        self.git_cmd(["push", self.remote_name, package])
        click.echo(f"Package {package} created")

    def _request_package_action(
        self, action: Type[PackageRequest | DeleteRequest], pkg: str
    ) -> None:
        ensured_upstream = self.gh_repo.get_root_repo()
        upstream_namespace = ensured_upstream.namespace
        issue_repo = self.gh_api.get_repo(upstream_namespace, self.gh_repo.repo_name)
        issue = issue_repo.create_issue(
            action.TITLE.value.format(package_name=pkg),
            action.BODY.value.format(
                user=self.gh_api.gh_user,
                package_name=pkg,
            ),
        )
        issue.add_to_labels(REQUEST_LABEL)

    def request_package(self, package_name: str) -> None:
        self._request_package_action(PackageRequest, package_name)

    def request_package_delete(self, package: str) -> None:
        self._request_package_action(DeleteRequest, package)



"""
These commands need to create LocalRepo -> no GH token required
"""
import os
import subprocess
from os import getcwd
from pathlib import Path
from shutil import which

import click
from click import ClickException

from alpa.config import MetadataConfig
from alpa.repository.branch import LocalRepoBranch, AlpaRepoBranch

from alpa.messages import NO_PRE_COMMIT

pkg_name = click.argument("name", type=str)


@click.command("show-history")
@click.option(
    "-o",
    "--oneline",
    is_flag=True,
    show_default=True,
    default=False,
    help="Each commit is displayed as one line",
)
def show_history(oneline: bool) -> None:
    """Show git history of current package"""
    params = []
    if oneline:
        params.append("--oneline")

    local_repo = LocalRepoBranch(Path(getcwd()))
    click.echo(local_repo.get_history_of_branch(local_repo.branch, params))


@click.command("switch")
@pkg_name
def switch(name: str) -> None:
    """Switch to specified package"""
    LocalRepoBranch(Path(getcwd())).switch_to_package(name)


@click.command("commit")
@click.option(
    "-m",
    "--message",
    type=str,
    default="",
    help="Your commit message not longer than 80 characters.",
)
@click.option("-n", "--no-verify", is_flag=True, help="Do not run pre-commit")
def commit(message: str, no_verify: bool) -> None:
    """Commit your changes in your package's repository"""
    if len(message) > 80:
        raise ClickException("Message longer than 80 characters")

    LocalRepoBranch(Path(getcwd())).commit(message, not no_verify)


@click.command("add")
@click.argument("to_add", type=str, required=True)
def add(to_add: str) -> None:
    """Add files to git history. Basically calls `git add <input>`"""
    LocalRepoBranch(Path(getcwd())).add(to_add)


def _skip_pre_commit_checks_for_non_rpm_os() -> None:
    process = subprocess.run(
        ["rpm", "--help"], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL
    )
    if process.returncode != 0:
        # no special pre-commit hooks for non-RPM OS :/
        disabled_checks = ["source0-uses-version-macro", "check-packit-file"]
        click.secho(
            "Warning! You don't have RPM based OS, these checks are "
            f"disabled: {disabled_checks}",
            fg="yellow",
        )
        os.environ["SKIP"] = ",".join(disabled_checks)


@click.command("push")
@click.option(
    "-p",
    "--pull-request",
    is_flag=True,
    show_default=True,
    default=False,
    help="This will create pull request on GitHub for you.",
)
@click.option("-n", "--no-verify", is_flag=True, help="Do not run pre-commit")
def push(pull_request: bool, no_verify: bool) -> None:
    """Pushes your commited changes to the Alpa repo so you can make PR"""
    if not no_verify:
        if which("pre-commit") is None:
            click.secho(NO_PRE_COMMIT, fg="red", err=True)
            return

        _skip_pre_commit_checks_for_non_rpm_os()
        ret = subprocess.run(["pre-commit", "run", "--all-files"])
        if ret.returncode != 0:
            # pre-commit already gives info about fail
            return

    repo_path = Path(getcwd())
    local_repo = LocalRepoBranch(repo_path)
    local_repo.push(local_repo.branch)

    if not pull_request:
        return

    alpa = AlpaRepoBranch(repo_path)
    pr = alpa.gh_repo.create_pr(
        title=f"[alpa-cli] Create update of package {local_repo.package}",
        body=(
            "This PR was created automatically via alpa-cli for "
            f"user {alpa.gh_api.gh_user}"
        ),
        source_branch=f"{local_repo.feat_branch}",
        target_branch=local_repo.package,
    )
    click.echo(f"PR#{pr.number} created. URL: {pr.html_url}")


@click.command("pull")
def pull() -> None:
    """Pull last recent changes of package you are on from Alpa repo"""
    local_repo = LocalRepoBranch(Path(getcwd()))
    local_repo.pull(local_repo.branch)


@click.command("list")
@click.option("-p", "--pattern", type=str, default="", help="Optional pattern to match")
def list_(pattern: str) -> None:
    """List all packages or packages matching regex"""
    for pkg in LocalRepoBranch(Path(getcwd())).get_packages(pattern):
        click.echo(pkg)


# TODO: please implement me
# @click.command("genspec")
# @click.option(
#     "--lang",
#     type=Choice(["python", "java"], case_sensitive=False),
#     required=True,
#     help="Choose the programming language for which the generator is designed",
# )
# @click.option(
#     "-t",
#     "--test",
#     default=False,
#     help=(
#         "Send package with generated spec file to "
#         "packit to test whether build will succeed."
#     ),
# )
# def genspec(lang: str, test: bool) -> None:
#     """This command uses some existing spec file generators for you"""
#     raise NotImplementedError("Not implemented yet (1.0 goal)")


@click.command("create-packit-config")
@click.option(
    "--override",
    is_flag=True,
    show_default=True,
    default=False,
    help="Override packit config file",
)
def create_packit_config(override: bool) -> None:
    """Creates packit config based on metadata.yaml file in package"""
    if not LocalRepoBranch(Path(getcwd())).create_packit_config(override):
        raise ClickException(
            "Packit file already exists. To override it use --override flag."
        )


@click.command("mockbuild")
@click.option(
    "--chroot",
    type=str,
    default="",
    help="Override chroots specified in metadata.yaml by one specific chroot",
)
def mockbuild(chroot: str) -> None:
    """
    Uses mock tool to build package. WARNING: works only on rpm-based systems.

    Builds for all chroots specified in metadata.yaml. Can be overriden by --chroot
     option which does build against one specified chroot.
    """
    from alpa.upstream_integration import UpstreamIntegration

    if chroot:
        chroots = [chroot]
    else:
        chroots = MetadataConfig.get_config().chroots

    UpstreamIntegration(Path(getcwd())).mockbuild(chroots)


@click.command("get-pkg-archive")
def get_pkg_archive() -> None:
    """Gets archive from package config. WARNING: works only on rpm-based systems."""
    from specfile import Specfile
    from alpa.upstream_integration import UpstreamIntegration

    specfile_path = None
    cwd = Path(getcwd())
    for file in cwd.iterdir():
        if str(file).endswith(".spec"):
            specfile_path = file

    if specfile_path is None:
        raise ClickException(f"No specfile found in {cwd}")

    specfile = Specfile(specfile_path)
    with specfile.sources() as sources:
        source0 = min(sources, key=lambda src: src.number)

    UpstreamIntegration.download_upstream_source(
        source0.expanded_location,
        f"{specfile.expanded_name}-{specfile.expanded_version}",
    )



"""
Config class for package metadata
"""


from os import getcwd
from pathlib import Path
from typing import Optional

from pydantic import EmailStr
from pydantic.dataclasses import dataclass
from yaml import safe_load

from alpa.constants import METADATA_FILE_NAMES
from alpa.config.base import Config

from alpa.config.alpa_repo import AlpaRepoConfig


@dataclass
class User:
    nick: str
    email: EmailStr


@dataclass
class Autoupdate:
    upstream_pkg_name: str
    anytia_backend: str  # TODO: anytia backend enum
    targets_notify_on_fail: set[str]


class MetadataConfig(Config):
    def __init__(
        self,
        autoupdate: Optional[Autoupdate],
        maintainers: list[User],
        targets: set[str],
        arch: set[str],
    ) -> None:
        self.autoupdate = autoupdate
        self.maintainers = maintainers
        self.targets = targets
        self.arch = arch

    @staticmethod
    def _fill_autoupdate_dataclass(d: dict) -> Autoupdate:
        targets_notify_on_fail = set(d.pop("targets_notify_on_fail", []))
        return Autoupdate(**d, targets_notify_on_fail=targets_notify_on_fail)

    @classmethod
    def _fill_metadata_from_dict(cls, d: dict) -> "MetadataConfig":
        autoupdate = d.get("autoupdate")
        autoupdate_dataclass = None
        if autoupdate is not None:
            autoupdate_dataclass = cls._fill_autoupdate_dataclass(autoupdate)

        maintainers = cls._check_for_mandatory_key(d, "maintainers", "metadata.yaml")
        users_list = []
        for maintainer_dict in maintainers:
            maintainer = cls._check_for_mandatory_key(
                maintainer_dict, "user", "metadata.yaml", "maintainers.user"
            )
            users_list.append(User(**maintainer))

        repo_config = AlpaRepoConfig.get_config()
        if repo_config.targets is None:
            targets = set(cls._check_for_mandatory_key(d, "targets", "metadata.yaml"))
        else:
            targets = set(d.get("targets", set())) | repo_config.targets

        if repo_config.arch is None:
            arch = set(d.get("arch", ["x86_64"]))
        else:
            arch = set(d.get("arch", [])) | repo_config.arch

        return MetadataConfig(
            autoupdate=autoupdate_dataclass,
            maintainers=users_list,
            targets=targets,
            arch=arch,
        )

    @classmethod
    def _load_metadata_config(cls, working_dir: Path) -> Optional["MetadataConfig"]:
        config_file_path = cls.get_config_file_path(working_dir, METADATA_FILE_NAMES)
        if config_file_path is None:
            return None

        with open(config_file_path) as meta_file:
            return cls._fill_metadata_from_dict(safe_load(meta_file.read()))

    @classmethod
    def get_config(cls, working_dir: Optional[Path] = None) -> "MetadataConfig":
        if working_dir is None:
            working_dir = Path(getcwd())

        metadata_data_class = cls._load_metadata_config(working_dir)
        if metadata_data_class is None:
            raise FileNotFoundError("No metadata file found in package")

        return metadata_data_class

    @property
    def chroots(self) -> list[str]:
        chroots = []
        for arch in self.arch:
            for distro in self.targets:
                chroots.append(f"{distro}-{arch}")
        return chroots



"""
Wrapper around git cmd using subprocess
"""
import asyncio
import logging
import subprocess
from dataclasses import dataclass
from typing import Optional


logger = logging.getLogger(__name__)


@dataclass
class GitCmdResult:
    stdout: str
    stderr: str
    retval: int

    @property
    def stderr_and_stdout(self) -> str:
        result = ""
        if self.stdout:
            result += self.stdout

        if not self.stderr:
            return result

        if result:
            result += "\n"

        result += self.stderr
        return result


class GitCMD:
    def __init__(self, cwd: str) -> None:
        self.git_root = cwd
        output = self.git_cmd(["rev-parse", "--show-toplevel"])
        if "not a git repository" in output.stderr:
            raise FileNotFoundError(output.stderr)

        self.git_root = output.stdout.strip()

    def git_cmd(
        self, arguments: list[str], cwd: Optional[str] = None
    ) -> "GitCmdResult":
        if cwd is None:
            context = self.git_root
        else:
            context = cwd

        logger.debug(
            f"Running git cmd: $ git {' '.join(arguments)}; in context {context}"
        )
        process = subprocess.run(
            ["git"] + arguments,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            cwd=context,
        )
        stdout = process.stdout.decode()
        stderr = process.stderr.decode()
        logger.debug(
            f"git cmd results: stdout: {stdout}; stderr: {stderr}; "
            f"retval: {process.returncode}"
        )
        return GitCmdResult(
            stdout=stdout.strip(),
            stderr=stderr.strip(),
            retval=process.returncode,
        )

    async def async_git_cmd(
        self, arguments: list[str], cwd: Optional[str] = None
    ) -> "GitCmdResult":
        if cwd is None:
            context = self.git_root
        else:
            context = cwd

        logger.debug(
            f"Running async git cmd: $ git {' '.join(arguments)}; in context {context}"
        )
        async_subprocess = await asyncio.create_subprocess_exec(
            "git",
            *arguments,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
            cwd=context,
        )
        stdout, stderr = await async_subprocess.communicate()
        logger.debug(
            f"async git cmd results: stdout: {stdout.decode()}; "
            f"stderr: {stderr.decode()}; retval: {async_subprocess.returncode}"
        )
        return GitCmdResult(
            stdout=stdout.decode().strip(),
            stderr=stderr.decode().strip(),
            retval=async_subprocess.returncode,  # type: ignore
        )



"""
Config class for Alpa repository.
"""


import subprocess
from enum import Enum
from pathlib import Path
from typing import Optional

from yaml import safe_load

from alpa.constants import ALPA_CONFIG_FILE_NAMES
from alpa.exceptions import AlpaConfException
from alpa.config.base import Config


class AlpaRepoType(str, Enum):
    branch = "branch"
    # TODO: in the future `subdirectory = "subdirectory"`


class AlpaRepoConfig(Config):
    def __init__(
        self,
        repo_type: AlpaRepoType,
        copr_owner: str,
        copr_repo: str,
        allow_foreign_contributing: bool = False,
        targets: Optional[set[str]] = None,
        arch: Optional[set[str]] = None,
    ) -> None:
        self.repo_type = repo_type
        self.copr_owner = copr_owner
        self.copr_repo = copr_repo

        # optional parameters
        self.allow_foreign_contributing = allow_foreign_contributing
        self.targets = targets
        self.arch = arch

    @classmethod
    def _config_from_dict(cls, d: dict) -> "AlpaRepoConfig":
        for mandatory_key in ["repo_type", "copr_owner", "copr_repo"]:
            cls._check_for_mandatory_key(d, mandatory_key, "alpa.yaml")

        targets = d.get("targets")
        if targets is not None:
            targets = set(targets)

        arch = d.get("arch")
        if arch is not None:
            arch = set(arch)

        return AlpaRepoConfig(
            repo_type=AlpaRepoType[d["repo_type"]],
            copr_owner=d["copr_owner"],
            copr_repo=d["copr_repo"],
            allow_foreign_contributing=d.get("allow_foreign_contributing", False),
            targets=targets,
            arch=arch,
        )

    @classmethod
    def _load_alpa_config(cls, pwd: Path) -> Optional["AlpaRepoConfig"]:
        config_file_path = cls.get_config_file_path(pwd, ALPA_CONFIG_FILE_NAMES)
        if config_file_path is None:
            return None

        with open(config_file_path) as alpa_config_file:
            return cls._config_from_dict(safe_load(alpa_config_file.read()))

    @classmethod
    def get_config(cls) -> "AlpaRepoConfig":
        process = subprocess.run(
            ["git", "rev-parse", "--show-toplevel"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        if process.returncode != 0:
            raise AlpaConfException(process.stderr)

        alpa_config = cls._load_alpa_config(Path(process.stdout.decode().strip()))
        if alpa_config is None:
            raise FileNotFoundError("No alpa config file in git root found")

        return alpa_config



"""
Wrapper aroung pygithub API since the documentation is awful..
"""
import logging
from os import getenv
from typing import Optional

import click
from click import UsageError
from github import Github, Issue, PullRequest, UnknownObjectException

from alpa.config.alpa_local import AlpaLocalConfig
from alpa.constants import GH_API_TOKEN_NAME, GH_WRITE_ACCESS
from alpa.messages import NO_GH_API_KEY_FOUND, RETURNING_CLONE_URL_MSG


logger = logging.getLogger(__name__)


class GithubRepo:
    def __init__(self, api: Github, namespace: str, repo_name: str) -> None:
        self.namespace = namespace
        self.repo_name = repo_name

        self._api = api
        self._repo = api.get_repo(f"{namespace}/{repo_name}")

    @property
    def clone_url(self) -> str:
        if self.has_write_access(self._api.get_user().login):
            return self._repo.ssh_url

        click.secho(
            RETURNING_CLONE_URL_MSG.format(
                user=self._api.get_user().login, repo=self._repo.full_name
            ),
            fg="bright_yellow",
        )
        return self._repo.clone_url

    @property
    def upstream_clone_url(self) -> Optional[str]:
        upstream = self.get_upstream()
        if upstream is None:
            return None

        return upstream.clone_url

    @property
    def is_fork(self) -> bool:
        return self._repo.fork

    @property
    def api_user(self) -> str:
        return self._api.get_user().login

    def has_write_access(self, user: str) -> bool:
        for collaborator in self._repo.get_collaborators():
            if collaborator.login != user:
                continue

            return (
                self._repo.get_collaborator_permission(collaborator) in GH_WRITE_ACCESS
            )

        return False

    def get_upstream(self) -> Optional["GithubRepo"]:
        if not self.is_fork:
            return None

        namespace, repo_name = self._repo.source.full_name.split("/")
        return GithubRepo(self._api, namespace, repo_name)

    def get_root_repo(self) -> "GithubRepo":
        root_repo = self.get_upstream()
        if root_repo is None:
            root_repo = self

        return root_repo

    def create_issue(self, title: str, body: str) -> Issue:
        return self.get_root_repo()._repo.create_issue(title, body)

    def create_pr(
        self, title: str, body: str, source_branch: str, target_branch: str = "main"
    ) -> PullRequest:
        return self.get_root_repo()._repo.create_pull(
            title, body, base=target_branch, head=source_branch
        )

    def pr_exists(self, id: int) -> bool:
        try:
            self._repo.get_pull(id)
            return True
        except UnknownObjectException:
            return False

    def get_issues(self, state: str, labels: list[str]) -> list[Issue]:
        return list(self._repo.get_issues(state=state, labels=labels))


class GithubAPI:
    def __init__(self, repo_name: str, gh_token: Optional[str] = None) -> None:
        if gh_token is not None:
            self._gh_api = Github(gh_token)
        else:
            self._gh_api = Github(self._get_access_token(repo_name))

    @property
    def gh_user(self) -> str:
        return self._gh_api.get_user().login

    @staticmethod
    def _get_access_token(repo_name: str) -> str:
        access_token_env = getenv(GH_API_TOKEN_NAME)
        if access_token_env is not None:
            return access_token_env

        # accessing config file here since it does not make sense to do it elsewhere,
        # but in the future the config will be useful in many places
        access_token_config = AlpaLocalConfig.get_config(repo_name)
        if access_token_config is None:
            raise UsageError(NO_GH_API_KEY_FOUND.format(token=GH_API_TOKEN_NAME))

        return access_token_config.gh_api_token

    def get_repo(self, namespace: str, repo_name: str) -> GithubRepo:
        logger.info(f"Trying to find repo: {namespace}/{repo_name}")
        return GithubRepo(self._gh_api, namespace, repo_name)



from alpa.repository.branch import AlpaRepoBranch, LocalRepoBranch
from alpa.repository.subdirectory import AlpaRepoSubdirectory, LocalRepoSubdirectory


__all__ = [
    AlpaRepoBranch.__name__,
    AlpaRepoSubdirectory.__name__,
    LocalRepoBranch.__name__,
    LocalRepoSubdirectory.__name__,
]



"""
File for really long messages.
"""


NO_GH_API_KEY_FOUND = (
    "No GitHub API key found. Please set the {token} environment variable and"
    " pass the GitHub API token to it or set it in alpa.yaml config file. \n"
    "For more information please see (link TODO)"
)

NO_PERMISSION_FOR_ALPA_REPO = (
    "You don't have write permission to this repository. Please fork this Alpa "
    "repository and work with your fork. Cloning a fork repo is done with "
    "`--fork` flag. e.g. `alpa clone --fork my-fork-repo`"
)
CLONED_REPO_IS_NOT_FORK = (
    "You are trying to clone repository which is not a fork! Clone a fork "
    "repository or do the clone command without `--fork` flag."
)
CLONED_REPO_IS_FORK = (
    "The url of GitHub repository is a fork. Please provide repository which "
    "is not a forked form Alpa repository or specify the clone command with "
    "the `--fork` flag."
)

NO_WRITE_ACCESS_ERR = (
    "You don't have write access to this repository. If you "
    "want to create new package, please use `alpa request-package "
    "[PACKAGE NAME]` command instead."
)

NOT_IN_PREDEFINED_STATE = (
    "Repository is not in predefined state. Seems like someone "
    "was playing with it. Please see TODO LINK to return this repo "
    "back to its predefined state."
)

RETURNING_CLONE_URL_MSG = (
    "User {user} does not have access to {repo} repository. "
    "Using https url for cloning instead of ssh url."
)

NO_PRE_COMMIT = (
    "pre-commit is not installed! Please install " "it via `pip install pre-commit`"
)



"""
Classes for communication with Alpa subdirectory type of git
"""


from pathlib import Path
from typing import Optional

from alpa.gh import GithubAPI

from alpa.repository.base import LocalRepo, AlpaRepo


class LocalRepoSubdirectory(LocalRepo):
    def __init__(self, repo_path: Path) -> None:
        super().__init__(repo_path)
        raise NotImplementedError("Please implement me.")

    def create_packit_config(self, override: bool) -> bool:
        raise NotImplementedError("Please implement me.")

    @property
    def package(self) -> str:
        raise NotImplementedError("Please implement me.")

    def _ensure_feature_branch(self) -> None:
        raise NotImplementedError("Please implement me.")

    def get_history_of_package(self, package: str) -> str:
        raise NotImplementedError("Please implement me.")

    def get_packages(self, regex: str = "") -> list[str]:
        raise NotImplementedError("Please implement me.")

    def switch_to_package(self, package: str) -> None:
        raise NotImplementedError("Please implement me.")


class AlpaRepoSubdirectory(AlpaRepo, LocalRepoSubdirectory):
    def __init__(self, repo_path: Path, gh_api: Optional[GithubAPI] = None) -> None:
        super().__init__(repo_path, gh_api)
        raise NotImplementedError("Please implement me.")

    def create_package(self, package: str) -> None:
        raise NotImplementedError("Please implement me.")

    def request_package_delete(self, package_name: str) -> None:
        raise NotImplementedError("Please implement me.")

    def delete_package(self, package: str) -> bool:
        raise NotImplementedError("Please implement me.")
